﻿Type=Service
Version=7.3
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
#Region  Service Attributes 
	#StartAtBoot: False
	#ExcludeFromLibrary: True
#End Region

Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.
	Dim multiport As CH341MultiPort
	Dim timeOut As Timer
	Dim timerTestSPI As Timer
	Dim timerInitSPI As Timer
	Dim logOn As Boolean = True
	Private manager As UsbManager
	Dim connectionStatus As Boolean = True
	Dim SPIon As Boolean = False
	
	' The length of the data packet supported by the CH341
	Dim mCH341_PACKET_LENGTH As Int = 32

	' The command packet of the UIO interface, starting from the next byte as the command stream
	Dim mCH341A_CMD_UIO_STREAM As Int = 0xAB

	' Command stream for UIO interface: input data D7-D0
	Dim mCH341A_CMD_UIO_STM_IN As Int = 0x00

	' Command flow of UIO interface: set I/O direction D5-D0, bit 5-bit 0 is direction data
	Dim mCH341A_CMD_UIO_STM_DIR As Int = 0x40

	' Command stream of UIO interface: output data D5-D0, bit 5-bit 0 is data
	Dim mCH341A_CMD_UIO_STM_OUT As Int = 0x80

	' Command stream of UIO interface: delay in microseconds, bit 5-bit 0 is the delay value
	Dim mCH341A_CMD_UIO_STM_US As Int = 0xC0
	
	' Command stream for UIO interface: command packet ends early
	Dim mCH341A_CMD_UIO_STM_END As Int = 0x20
End Sub

Sub Service_Create
	'This is the program entry point.
	'This is a good place to load resources that are not specific to a single activity.
	multiport.initialize(GetContext, Application.PackageName, logOn)
	timeOut.Initialize("timeOut", 1000)
	timeOut.Enabled = True
	manager.Initialize
	
	timerInitSPI.Initialize("timerInitSPI", 5000)

	timerTestSPI.Initialize("timerTestSPI", 1000)
End Sub

Sub Service_Start (StartingIntent As Intent)
	multiportResume
End Sub

Sub Service_TaskRemoved
	'This event will be raised when the user removes the app from the recent apps list.
End Sub

'Return true to allow the OS default exceptions handler to handle the uncaught exception.
Sub Application_Error (Error As Exception, StackTrace As String) As Boolean
	Return True
End Sub

Sub Service_Destroy
	timerInitSPI.Enabled = False
	timerTestSPI.Enabled = False
	multiport.finalize
End Sub

Sub multiportResume
	multiport.resume
	
	If multiport.CH341SystemInit < 0 Then
		Log("System init error")
	End If
	
	If logOn Then
		Log("isConnected: " & multiport.isConnected)
		
		Log("CH341GetPIDVID:")
		Dim VIDPID(,) As Int = multiport.CH341GetPIDVID
		
		For i = 0 To VIDPID.Length - 1
			Log(i & " - VID: " &  VIDPID(i, 0) & " / PID: " &  VIDPID(i, 1))
		Next
				
		Log("CH341GetBufSizeValue: " & multiport.CH341GetBufSizeValue)
		Log("CH341GetChipVersion: " & multiport.CH341GetChipVersion)
		Log("CH341GetDescription: " & multiport.CH341GetDescription)
		Log("CH341GetDevName: " & multiport.CH341GetDevName)
		Log("CH341GetLibraryVendor: " & multiport.CH341GetLibraryVendor)
		Log("CH341GetReadTimeOutValue: " & multiport.CH341GetReadTimeOutValue)
		Log("CH341GetWriteTimeOutValue: " & multiport.CH341GetWriteTimeOutValue)
	End If
	
	timerInitSPI.Enabled = True
	
	Log("MegaSpiEnable:" & MegaSpiEnable)
	
'	If Not(MegaSpiEnable) Then	' Initialize the SPI of the CH341
'		If logOn Then
'			Log("MegaSpiEnable: failed")
'		End If
'	End If
End Sub

Sub timeOut_tick
	Try
		If manager.GetDevices.Length = 0 Then
			If connectionStatus Then
				connectionStatus = False
				Log("No connected usb devices.")
			End If
		Else
			If Not(connectionStatus) Then
				connectionStatus = True
			End If
			Dim device As UsbDevice = manager.GetDevices(0) 'assuming that there is exactly one device
			If manager.HasPermission(device) = False Then
				ToastMessageShow("Please allow connection and click again.", True)
				manager.RequestPermission(device)
			Else
				If Not(multiport.isConnected) Then
					multiport.OpenUsbDevice(device)
					multiportResume
				End If
			End If
		End If
	Catch
		Log(LastException)
	End Try			
End Sub

Sub MegaSpiEnable As Int
	
	Dim mBuffer(mCH341_PACKET_LENGTH) As Byte
	
	Dim i As Int = 0
	
	mBuffer(i) = mCH341A_CMD_UIO_STREAM					' Command code
	i = i + 1
	mBuffer(i) = Bit.Or(mCH341A_CMD_UIO_STM_OUT, 0x00)	' default status: all 0
	i = i + 1
	mBuffer(i) = Bit.Or(mCH341A_CMD_UIO_STM_DIR, 0x2B)	' D0 & D1 & D3 & D5 output, other input
	i = i + 1
	mBuffer(i) = Bit.Or(mCH341A_CMD_UIO_STM_US, 32)		' Delay 32 microseconds
	i = i + 1
	mBuffer(i) = mCH341A_CMD_UIO_STM_END				' The current command packet ends prematurely
	i = i + 1

	Return multiport.WriteData(mBuffer, i, 10)
End Sub

Sub SetChipSelectSS(ss As Int) As Int
'	i = 0;
'	mBuffer[ i++ ] = mCH341A_CMD_UIO_STREAM;            // 命令码
'	mBuffer[ i++ ] = mCH341A_CMD_UIO_STM_OUT |0x00;     // D1=0
'	mBuffer[ i++ ] = mCH341A_CMD_UIO_STM_END;
'	If(CH341WriteData( index, mBuffer, &i ) == False )
'	Return FALSE;

	Dim mBuffer(mCH341_PACKET_LENGTH) As Byte
	
	Dim i As Int = 0
	
	mBuffer(i) = mCH341A_CMD_UIO_STREAM					' Command code
	i = i + 1
	mBuffer(i) = Bit.Or(mCH341A_CMD_UIO_STM_OUT, Bit.And(ss, 0x01))	' default status: D0=SS, other 0
	i = i + 1
	mBuffer(i) = mCH341A_CMD_UIO_STM_END				' The current command packet ends prematurely
	i = i + 1

	Return multiport.WriteData(mBuffer, i, 10)
End Sub

Sub timerInitSPI_tick
	SPIon = True
	
	multiport.CH341SetStream(0x81) '0x81
		
	timerInitSPI.Enabled = False
	timerTestSPI.Enabled = True
End Sub

Sub timerTestSPI_tick
	timerTestSPI.Enabled = False
	
	If Not(connectionStatus) Then
		timerTestSPI.Enabled = True
		Return
	End If
	If Not(SPIon) Then
		timerTestSPI.Enabled = True
		Log("SPI init error")
		Return
	End If
'	If Not(multiport.SPI_WriteEnable) Then
'		Log("SPI write enable error")
'		Return
'	End If
	
	Dim bc As ByteConverter
	
	Dim dataBuff(128) As Byte
	
	Dim checkFail As Boolean = False
	
	Dim millis0 As Long
	Dim millis1 As Long
	Dim str As String = DateTime.GetSecond(DateTime.Now) & " - Test SPI Write 0123456789" & Chr(13) & Chr(10)
	
	
	
	'For i = 0 To 99
	
	Dim str2 As String = ""
	
	Do While (str2.Length + str.Length) < 128
		str2 = str2 & str
	Loop
	
	Log("str2 length: " & str2.Length)

	dataBuff = bc.StringToBytes(str2, "UTF-8")
			
'	Dim bc As ByteConverter
'	Dim str As String = ""
'	For j = 0 To dataBuff.Length -2
'		str = str & bc.HexFromBytes(Array As Byte(dataBuff(j))) & " "
'	Next
'	str = str & bc.HexFromBytes(Array As Byte(dataBuff(dataBuff.Length - 1)))
'	Log(str)

	millis0 = DateTime.Now

	'multiport.SetDeviceType(0) ' flash cmd codes
	SetChipSelectSS(0)
	checkFail = Not(multiport.CH341StreamSPI4(0, str2.Length, dataBuff))
	SetChipSelectSS(1)
	
	millis1 = DateTime.Now
	
	Log("Time to send: " & (millis1 - millis0) & "ms")
	
'	str = ""
'	For j = 0 To dataBuff.Length -2
'		str = str & bc.HexFromBytes(Array As Byte(dataBuff(j))) & " "
'	Next
'	str = str & bc.HexFromBytes(Array As Byte(dataBuff(dataBuff.Length - 1)))
'	Log(str)

	If checkFail Then
		'i = 999 ' break
	End If
		
	'Sleep(0)
	'Next
	
	
			
	
	
	If checkFail Then
		Log(DateTime.GetSecond(DateTime.Now) & " - SPI write error")
	Else
		Log(DateTime.GetSecond(DateTime.Now) & " - SPI write done")
	End If
	
	timerTestSPI.Enabled = True

End Sub

Sub SPI_write(buf() As Byte) As Boolean
	multiport.SetDeviceType(0)
	Return multiport.CH341StreamSPI4(3, buf.Length, buf)
End Sub

'Sub SPI_read(addr As Int, length As Int) As Byte()
	'multiport.SPI_Init
	'Return multiport.SPI_FlashReadBlock(addr, length)
'End Sub

Sub GetContext As JavaObject
	Return GetBA.GetField("context")
End Sub

Sub GetBA As JavaObject
	Dim jo As JavaObject
	Dim cls As String = Me
	cls = cls.SubString("class ".Length)
	jo.InitializeStatic(cls)
	Return jo.GetFieldJO("processBA")
End Sub